
'''
    This file is part of batti, a battery monitor for the system tray.
    Copyright (C) 2010  Arthur Spitzer

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
'''

from Battery import DeviceKitBattery
from Battery import UPowerBattery
import dbus
from dbus.exceptions import DBusException


class PowerBackend(object):
    
    def set_right_popup_menu_action(self, action):
        pass
    
    def can_suspend(self):
        pass
    
    def can_hibernate(self):
        pass

    def suspend(self):
        pass
    
    def hibernate(self):
        pass
    

class DeviceKitBackend(PowerBackend):
    
    dbus_service = 'org.freedesktop.DeviceKit.Power'
    dbus_object = '/org/freedesktop/DeviceKit/Power'
    dbus_interface = 'org.freedesktop.DeviceKit.Power'
    device_interface = 'org.freedesktop.DeviceKit.Power.Device'
    bat_type = 2
    
    def __init__(self):
        self.__rmenu = None
        
        self.__bus = dbus.SystemBus()
        dkit_obj = self.__bus.get_object(self.dbus_service, self.dbus_object)
        self.dkit = dbus.Interface(dkit_obj, self.dbus_interface)
        devices = self.dkit.EnumerateDevices()
        
        self.__batteries = {}
        for dev in devices:
            (prop_iface, dev_iface) = self.__get_battery(dev)
            type = prop_iface.Get(self.device_interface, 'type')
            if type == self.bat_type:
                power_bat = DeviceKitBattery(prop_iface, dev_iface)
                self.__batteries[dev] = power_bat
                power_bat.set_left_popup_menu_action(self.__mc_action)
                
        
        self.__bus.add_signal_receiver(self.__device_added, 'DeviceAdded',
             self.dbus_interface, self.dbus_service, self.dbus_object)
        self.__bus.add_signal_receiver(self.__device_removed, 'DeviceRemoved',
             self.dbus_interface, self.dbus_service, self.dbus_object)
        self.__mc_action = None
        

        properties = dbus.Interface(dkit_obj, 'org.freedesktop.DBus.Properties')

        if properties.Get(self.dbus_interface, 'can-suspend'):
            self.__can_suspend = True
        else:
            self.__can_suspend = False
           
        if properties.Get(self.dbus_interface, 'can-hibernate'):
            self.__can_hibernate = True
        else:
            self.__can_hibernate = False
            
    
    def __mc_action(self, widget, event, data=None):
        if not self.__mc_action is None:
            self.__mc_action(widget, event, data)
            
         
    def can_suspend(self):   
        return self.__can_suspend

    def can_hibernate(self):
        return self.__can_hibernate

    def suspend(self):
        self.dkit.Suspend()

    def hibernate(self):
        self.dkit.Hibernate()


    def __get_battery(self, udi):
        battery_obj = self.__bus.get_object(self.dbus_service, udi)
        prop_iface = dbus.Interface(battery_obj, 'org.freedesktop.DBus.Properties')
        dev_iface = dbus.Interface(battery_obj, self.device_interface)
        return (prop_iface, dev_iface)
            
    
    def set_right_popup_menu_action(self, action):
        self.__mc_action = action
            
            
    def __device_added(self, udi):
        (prop_iface, dev_iface) = self.__get_battery(udi)
        try:
            type = prop_iface.Get(self.device_interface, 'type')
            if type == self.bat_type:
                power_bat = DeviceKitBattery(prop_iface, dev_iface)
                power_bat.set_left_popup_menu_action(self.__mc_action)
                power_bat.update()
                self.__batteries[udi] = power_bat
        except DBusException:
            pass
    
    
    def __device_removed(self, udi):
        try:
            halbat = self.__batteries.pop(udi)
            halbat.destroy()
        except KeyError:
            pass
    
    
    def update_batteries(self, startup=False):
        for bat in self.__batteries.itervalues():
            notvis = bat.get_notification_enabled()
            if startup:
                bat.set_notification_enabled(False)
            bat.update()
            bat.set_notification_enabled(notvis)
    
    
    
class UPowerBackend(PowerBackend):
    
    dbus_service = 'org.freedesktop.UPower'
    dbus_object = '/org/freedesktop/UPower'
    dbus_interface = 'org.freedesktop.UPower'
    device_interface = 'org.freedesktop.UPower.Device'
    bat_type = 2
    
    def __init__(self):
        self.__rmenu = None
        
        self.__bus = dbus.SystemBus()
        iface = self.__get_interface()
        devices = iface.EnumerateDevices()
        
        self.__batteries = {}
        for dev in devices:
            (prop_iface, dev_iface) = self.__get_battery(dev)
            type = prop_iface.Get(self.device_interface, 'type')
            if type == self.bat_type:
                power_bat = UPowerBattery(prop_iface, dev_iface)
                self.__batteries[dev] = power_bat
                power_bat.set_left_popup_menu_action(self.__mc_action)
                
        
        self.__bus.add_signal_receiver(self.__device_added, 'DeviceAdded',
             self.dbus_interface, self.dbus_service, self.dbus_object)
        self.__bus.add_signal_receiver(self.__device_removed, 'DeviceRemoved',
             self.dbus_interface, self.dbus_service, self.dbus_object)
        self.__mc_action = None
        
        properties = dbus.Interface(iface, 'org.freedesktop.DBus.Properties')

        if properties.Get(self.dbus_interface, 'CanSuspend'):
            self.__can_suspend = True
        else:
            self.__can_suspend = False
        
        if properties.Get(self.dbus_interface, 'CanHibernate'):
            self.__can_hibernate = True
        else:
            self.__can_hibernate = False
    
    
    def __get_interface(self):
        dkit_obj = self.__bus.get_object(self.dbus_service, self.dbus_object)
        return dbus.Interface(dkit_obj, self.dbus_interface)
    
    
    def __mc_action(self, widget, event, data=None):
        if not self.__mc_action is None:
            self.__mc_action(widget, event, data)
            
         
    def can_suspend(self):   
        return self.__can_suspend and self.__get_interface().SuspendAllowed()

    def can_hibernate(self):
        return self.__can_hibernate and self.__get_interface().HibernateAllowed()

    def suspend(self):
        self.__get_interface().Suspend()

    def hibernate(self):
        self.__get_interface().Hibernate()


    def __get_battery(self, udi):
        battery_obj = self.__bus.get_object(self.dbus_service, udi)
        prop_iface = dbus.Interface(battery_obj, 'org.freedesktop.DBus.Properties')
        dev_iface = dbus.Interface(battery_obj, self.device_interface)
        return (prop_iface, dev_iface)
            
    
    def set_right_popup_menu_action(self, action):
        self.__mc_action = action
            
            
    def __device_added(self, udi):
        (prop_iface, dev_iface) = self.__get_battery(udi)
        try:
            type = prop_iface.Get(self.device_interface, 'type')
            if type == self.bat_type:
                power_bat = UPowerBattery(prop_iface, dev_iface)
                power_bat.set_left_popup_menu_action(self.__mc_action)
                power_bat.update()
                self.__batteries[udi] = power_bat
        except DBusException:
            pass
    
    
    def __device_removed(self, udi):
        try:
            bat = self.__batteries.pop(udi)
            bat.destroy()
        except KeyError:
            pass
    
    
    def update_batteries(self, startup=False):
        for bat in self.__batteries.itervalues():
            notvis = bat.get_notification_enabled()
            if startup:
                bat.set_notification_enabled(False)
            bat.update()
            bat.set_notification_enabled(notvis)
    




'''
    This file is part of batti, a battery monitor for the system tray.
    Copyright (C) 2010  Arthur Spitzer

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
'''

import gettext
import gtk

from Notificator import Notificator


_ = lambda msg: gettext.dgettext('batti', msg)

# battery power state
( 
    STATE_UNKNOWN,
    STATE_CHARGING,
    STATE_DISCHARGING,
    STATE_EMPTY,
    STATE_CHARGED,
) = range(5)


class BatteryInfo(object):
    
    def __init__(self, present=True, state=STATE_UNKNOWN, percentage=0, time=0):
        self._present = present
        self._state = state
        self._percentage = percentage
        self._time = time


class Battery(object):
    
    def __init__(self):
        self.__systray = gtk.StatusIcon()
        self.__notifer = Notificator()
        
        self.__shown_charging = False
        self.__shown_on_bat = False
        self.__shown_bat_low = False
        self.__shown_bat_critical = False
        self.__shown_bat_charged = False
        self.__show_notify = True
        
        self._set_icon_visibility(True)
        
        
    def __del__(self):
        self._set_icon_visibility(False)
        del self.__systray
    
    
    def destroy(self):
        self.__del__()
    
        
    def update(self):
        pass
    
    
    def set_left_popup_menu_action(self, menu_action):
        self.__systray.connect('button_press_event', menu_action)
  
  
    def set_value(self, info):
        
        self._set_icon_visibility(info._present)
        
        if info._time == 0:
            tooltip = _("Battery level: %s%%") % (info._percentage)
            notification = tooltip
        else:
            time_str = self._str_time(info._time)
            tooltip = _('Battery level: %(level)s%%\nProviding power for approximately %(time)s') % {'level': info._percentage, 'time': time_str}
            notification = _('Power for approximately <b>%s</b> remaining') % time_str
        
        if info._state == STATE_CHARGING:
            self.__systray.set_blinking(False)
            self.__systray.set_tooltip(_('Charging battery\nBattery level: %s%%') % info._percentage)
            if info._percentage > 80:
                icon = 'battery-full-charging'
            elif info._percentage > 40:
                icon = 'battery-good-charging'
            elif info._percentage > 20:
                icon = 'battery-low-charging'
            else:
                icon = 'battery-caution-charging'
            self.__set_tray_icon(icon)
            
            self.__shown_bat_charged = False
            if not self.__shown_charging:
                self.__shown_bat_critical = False
                self.__shown_bat_low = False
                self.__shown_charging = True
                self.__shown_on_bat = False
                self._notify(False, icon, _('Charging battery'), notification)
                
        elif info._state == STATE_DISCHARGING:
            self.__systray.set_tooltip(tooltip)
            if info._percentage > 80:
                icon = 'battery-full'
            elif info._percentage > 40:
                icon = 'battery-good'
            elif info._percentage > 20:
                icon = 'battery-low'
            elif info._percentage > 5:
                icon = 'battery-caution'
                if not self.__shown_bat_low:
                    self._notify(True, icon, _('Low battery level'), notification)
                    self.__shown_bat_low = True
            else:
                icon = 'battery-empty'
                self.__systray.set_blinking(True)
                if not self.__shown_bat_critical:
                    self._notify(True, icon, _('Critical battery level'), notification)
                    self.__shown_bat_critical = True
            self.__set_tray_icon(icon)
            self.__shown_bat_charged = False
            if not self.__shown_on_bat:
                self._notify(False, icon, _("Discharging battery"), notification)
                self.__shown_charging = False
                self.__shown_on_bat = True
                    
        elif info._state == STATE_CHARGED:
            self.__systray.set_tooltip(_('Battery charged\n%s') % tooltip)
            self.__set_tray_icon('battery-full')
            self.__systray.set_blinking(False)
            if not self.__shown_bat_charged:
                self._notify(False, 'battery-full-charging', _('Battery charged'), notification)
                self.__shown_bat_charged = True
                     
    
    def __set_tray_icon(self, icon_name):
        self.__systray.set_from_icon_name(icon_name)
    
    
    def _set_icon_visibility(self, visible):
        self.__systray.set_visible(visible)
    
        
    def _notify(self, urgent, icon, subject, msg):
        if self.__show_notify:
            posrect = self.__systray.get_geometry()[1]
            posx = posrect.x + posrect.width/2
            posy = posrect.y + posrect.height
            self.__notifer.setPosition(posx, posy)
            if urgent:
                self.__notifer.show_urgent(icon, subject, msg)
            else: 
                self.__notifer.show(icon, subject, msg)
    
    
    def set_notification_enabled(self, enabled):
        self.__show_notify = enabled
        
        
    def get_notification_enabled(self):
        return self.__show_notify
    
    
    def _str_time(self, seconds):
        if seconds < 0:
            return _('unknown time')
       
        minutes = seconds / 60
        hours = minutes / 60
        minutes = minutes % 60                    
       
        #FIXME: The string below needs to be i18n-ized properly
        return self._format_time(hours, _('Hour'), _('Hours')) + " " + self._format_time(minutes, _('Minute'), _('Minutes'))


    def _format_time(self, time, singular, plural):
        if time == 0:
            return ""
        elif time == 1:
            return "1 %s" % singular
        else:
            return "%s %s" % (time, plural)
        
 


class DeviceKitBattery(Battery):
    
    dbus_iface = 'org.freedesktop.DeviceKit.Power.Device'
    
    def __init__(self, property_iface, device_iface):
        Battery.__init__(self)
        self.__properties = property_iface
        self.__device = device_iface
        self.__signal_id = self.__device.connect_to_signal('Changed', self.__on_property_modified)
     
     
    def __del__(self):
        self.__signal_id.remove()
        Battery.__del__(self)
    
    
    def update(self):
        self.__on_property_modified()
    
    
    def __on_property_modified(self):
        
        info = BatteryInfo()
        
        
        info._present = self.__properties.Get(self.dbus_iface, 'is-present')

        #XXX: check if the battery is rechargable first
        state = self.__properties.Get(self.dbus_iface, 'state')
        if state == 1:
            info._state = STATE_CHARGING
        elif state == 2:
            info._state = STATE_DISCHARGING
        else:
            info._state = STATE_UNKNOWN
        
        info._time = self.__properties.Get(self.dbus_iface, 'time-to-empty')
   
        charge_level = self.__properties.Get(self.dbus_iface, 'percentage')
        info._percentage = int(float(str(charge_level)))

        self.set_value(info)




class UPowerBattery(Battery):
    
    dbus_iface = 'org.freedesktop.UPower.Device'
    
    def __init__(self, property_iface, device_iface):
        Battery.__init__(self)
        self.__properties = property_iface
        self.__device = device_iface
        self.__signal_id = self.__device.connect_to_signal('Changed', self.__on_property_modified)
     
     
    def __del__(self):
        self.__signal_id.remove()
        Battery.__del__(self)
    
    
    def update(self):
        self.__on_property_modified()
    
    
    def __on_property_modified(self):
        
        info = BatteryInfo()
        
        present = self.__properties.Get(self.dbus_iface, 'IsPresent')
        self._set_icon_visibility(present)
        info._present = present

        state = self.__properties.Get(self.dbus_iface, 'State')

        if state == 1:
            info._state = STATE_CHARGING
        elif state == 2:
            info._state = STATE_DISCHARGING
        elif state == 3:
            info._state = STATE_EMPTY
        elif state == 4:
            info._state = STATE_CHARGED
        else:
            info._state = STATE_UNKNOWN
        
        info._time = self.__properties.Get(self.dbus_iface, 'TimeToEmpty')
   
        precise_charge = self.__properties.Get(self.dbus_iface, 'Percentage')
        info._percentage = int(float(str(precise_charge)))
        
        self.set_value(info)




'''
    This file is part of batti, a battery monitor for the system tray.
    Copyright (C) 2010  Arthur Spitzer

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
'''

from dbus.exceptions import DBusException
from dbus.mainloop.glib import DBusGMainLoop
import gettext
import gtk
import sys

import Constants
from PowerBackend import DeviceKitBackend
from PowerBackend import UPowerBackend

_ = lambda msg: gettext.dgettext(Constants.NAME, msg)

class BatteryMonitor(object):
    def __init__(self):
        DBusGMainLoop(set_as_default=True)
        self.__rmenu = None
        self.__lmenu = None
        try:
            self.__backend = UPowerBackend()
            self.__backend.set_right_popup_menu_action(self.__mc_action)
        except DBusException as e_upower:
            try:
                self.__backend = DeviceKitBackend()
                self.__backend.set_right_popup_menu_action(self.__mc_action)
            except DBusException as e_devkit:
                sys.stderr.write(""" 
Neither UPower nor DeviceKit.Power could be initialized!
This can have multiple reasons.
Here is the error for UPower:

    %s

And this is the error for DeviceKit.Power:

    %s

""" % (e_upower, e_devkit))
                self.close(None)
                
        
    
    def __suspend(self, button):
        try:
            self.__backend.suspend()
        except DBusException as e:
            self.__handle_exception(e)
        
        
    def __hibernate(self, button):
        try:
            self.__backend.hibernate()
        except DBusException as e:
            self.__handle_exception(e)
    
    
    def __handle_exception(self, exception):
        sys.stderr.write('%s' % exception)
    
    
    def __get_left_click_menu(self):
        if self.__lmenu is None:
            self.__lmenu = gtk.Menu()
            if self.__backend.can_suspend():
                suspend_item = gtk.ImageMenuItem(_('Suspend'))
                suspend_icon = gtk.image_new_from_icon_name('batti-suspend', gtk.ICON_SIZE_MENU)
                suspend_item.set_image(suspend_icon)
                suspend_item.connect('activate', self.__suspend)
                self.__lmenu.append(suspend_item)
            if self.__backend.can_hibernate():
                hibernate_item = gtk.ImageMenuItem(_('Hibernate'))
                hibernate_icon = gtk.image_new_from_icon_name('batti-hibernate', gtk.ICON_SIZE_MENU)
                hibernate_item.set_image(hibernate_icon)
                hibernate_item.connect('activate', self.__hibernate)
                self.__lmenu.append(hibernate_item)
            self.__lmenu.show_all()
        return self.__lmenu
    
    
    def __get_right_click_menu(self):
        if self.__rmenu is None:
            self.__rmenu = gtk.Menu()
            about_menu = gtk.ImageMenuItem(gtk.STOCK_ABOUT)
            about_menu.connect('activate', self.about)
            exit_menu = gtk.ImageMenuItem(gtk.STOCK_QUIT)
            exit_menu.connect('activate', self.close)
            self.__rmenu.append(about_menu)
            self.__rmenu.append(exit_menu)
            self.__rmenu.show_all()
        return self.__rmenu
    
        
    def __mc_action(self, widget, event, data=None):
        if event.button == 1:
            if self.__backend.can_suspend() or self.__backend.can_hibernate():
                self.__get_left_click_menu().popup(None, None, None, event.button, event.time)
        elif event.button == 3:
            self.__get_right_click_menu().popup(None, None, None, event.button, event.time)
        
        
    def close(self,button):
        # Check if we're already in gtk.main loop
        if gtk.main_level() > 0:
            gtk.main_quit()
        else:
            exit(1)


    def about(self, button):
        about_dg = gtk.AboutDialog()
        about_dg.set_name("Batti")
        about_dg.set_program_name(Constants.NAME)
        about_dg.set_version(Constants.VERSION)
        about_dg.set_comments(_("Battery Monitor"))
        about_dg.set_license(Constants.LICENSE_TEXT)
        about_dg.set_authors(['%s <%s>' % (Constants.AUTHOR, Constants.AUTHOR_EMAIL)])
        about_dg.connect("response", lambda d, r: d.destroy())
        about_dg.show()
            
    def main(self):
        try:
            self.__backend.update_batteries(startup=True)
            gtk.main()
        except KeyboardInterrupt:
            pass

if __name__ == "__main__":
    monitor = BatteryMonitor()
    monitor.main()



#!/usr/bin/env python
'''
    This file is part of batti, a battery monitor for the system tray.
    Copyright (C) 2010  Arthur Spitzer

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
'''

import glob
import os
import sys
from distutils.core import setup
from distutils.command.install_data import install_data
from distutils.command.build import build
from distutils.dep_util import newer
from distutils.log import info
from subprocess import call
from src import Constants

PO_DIR = 'po'
MO_DIR = os.path.join('build', 'mo')
ICON_DIR = os.path.join("data", "icons")

#Create an array with all the images
ICONS = []
current_dir = os.getcwd()
os.chdir(ICON_DIR)
png_paths = glob.glob("??x??/*")
svg_paths = glob.glob("scalable/*")
os.chdir(current_dir)
for filepath in png_paths:
    targetpath = os.path.join("share/icons/hicolor", filepath)
    sourcepath = "%s/%s/*.png" % (ICON_DIR, filepath)
    ICONS.append((targetpath, glob.glob(sourcepath)))
for filepath in svg_paths:
    targetpath = os.path.join("share/icons/hicolor/", filepath)
    sourcepath = "%s/%s/*.svg" % (ICON_DIR, filepath)
    ICONS.append((targetpath, glob.glob(sourcepath)))
    
    
class BuildLocales(build):
  def run(self):
    build.run(self)

    for po in glob.glob(os.path.join(PO_DIR, '*.po')):
      lang = os.path.basename(po[:-3])
      mo = os.path.join(MO_DIR, lang, Constants.NAME + '.mo')

      directory = os.path.dirname(mo)
      if not os.path.exists(directory):
        os.makedirs(directory)

      if newer(po, mo):
        info('compiling %s -> %s' % (po, mo))
        try:
          rc = call(['msgfmt', po, '-o', mo])
          if rc != 0:
            raise Warning, "msgfmt returned %d" % rc
        except Exception, e:
          print "Building gettext files failed."
          print "%s: %s" % (type(e), str(e))
          sys.exit(1)


class InstallLocales(install_data):
  def run(self):
    self.data_files.extend(self._find_mo_files())
    install_data.run(self)

  def _find_mo_files(self):
    data_files = []
    for mo in glob.glob(os.path.join(MO_DIR, '*', Constants.NAME + '.mo')):
        lang = os.path.basename(os.path.dirname(mo))
        dest = os.path.join('share', 'locale', lang, 'LC_MESSAGES')
        data_files.append((dest, [mo]))
    return data_files



setup(name = Constants.NAME,
    version = Constants.VERSION, 
    description = Constants.DESCRIPTION,
    author = Constants.AUTHOR,
    author_email = Constants.AUTHOR_EMAIL,
    url = Constants.URL,
    license = 'GNU GPLv2',
    platforms = ['Linux'],
    classifiers = [
        'Development Status :: 5 - Production/Stable',
        'Environment :: X11 Applications :: GTK',
        'Intended Audience :: End Users/Desktop',
        'License :: OSI Approved :: GNU General Public License (GPL)',
        'Natural Language :: English',
        'Operating System :: POSIX :: Linux',
        'Programming Language :: Python :: 2.5',
        'Topic :: System :: Hardware',
        'Topic :: Utilities',
    ],
    package_dir = {Constants.NAME: 'src'},
    packages = [Constants.NAME],
    scripts = [Constants.NAME],
    cmdclass = {'build': BuildLocales, 'install_data': InstallLocales},
    data_files = [
        ('share/applications/', ['data/%s.desktop' % Constants.NAME])
    ]+ICONS
)





'''
    This file is part of batti, a battery monitor for the system tray.
    Copyright (C) 2010  Arthur Spitzer

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
'''

import dbus
from dbus.exceptions import DBusException

class Notificator:
    
    def __init__(self):
        try:
            bus = dbus.SessionBus()
            obj = bus.get_object('org.freedesktop.Notifications', '/org/freedesktop/Notifications')
            self.__notify = dbus.Interface(obj, 'org.freedesktop.Notifications')
            self.__last_id = dbus.UInt32(0)
            self.__posx = -1
            self.__posy = -1
            self.__positioned = True
            self.__duration = 3000
            self.__name = 'batti'
            self.__check_capabilities()
        except DBusException:
            self.__notify = None
            self.__positioned = False


    def __check_capabilities(self):
        info = self.__notify.GetServerInformation()
        if info[0] == 'notify-osd':
            self.__positioned = False


    def __show_positioned(self):
        if self.__positioned:
            return (self.__posx >= 0 and self.__posy >= 0)
        else:
            return False


    def __show(self, icon, subject, msg, urgent):
        if self.__notify is not None:
            hints = {'urgency':dbus.Byte(urgent), 'desktop-entry':dbus.String('battery-monitor')}
            if( self.__show_positioned() ):
                hints['x'] = self.__posx
                hints['y'] = self.__posy
            self.__last_id = self.__notify.Notify(self.__name, self.__last_id, icon, subject, msg, [], hints, self.__duration)


    def show(self, icon, subject, msg):
        self.__show(icon, subject, msg, 1)


    def show_urgent(self, icon, subject, msg):
        self.__show(icon, subject, msg, 2)


    def close(self):
        if (self.__notify is not None) and self.__last_id:
            self.__notify.CloseNotification(self.__last_id)


    def setPosition(self, x, y):
        self.__posx = x
        self.__posy = y
    
    
    def removePosition(self):
        self.__posx = -1
        self.__posy = -1
        
    def setDuration(self, milSec):
        ''' Set the duration on a notification with milSec milliseconds '''
        self.__duration = milSec
        
        